from amaranth import Array, Module, Cat, Signal, Assert, unsigned
from amaranth.build import Platform
from amaranth.lib import wiring, data
from amaranth.lib.wiring import In, Out
from amaranth.lib.memory import Memory, WritePort
from amaranth.sim import Simulator

from ..hub75 import Hub75Coordinator, Hub75StringDriver, Rgb666Layout


def test_stringdriver():
    # the string driver test must
    # 1. finish
    # 2. strobe through all of the data in the array
    # 3. slice the correct bit from the data.
    m = Module()
    m.submodules.dut = dut = Hub75StringDriver()
    m.submodules.mem = mem = Memory(
        shape=data.ArrayLayout(Rgb666Layout, 2), depth=128, init=[]
    )
    port = mem.read_port()

    wiring.connect(m, port, dut.bram_port)

    async def testbench(ctx):
        # select a bit, strobe start, read values, test against known.
        ctx.set(dut.bcm_select, 5)
        ctx.set(dut.start, 1)
        await ctx.tick()
        ctx.set(dut.start, 0)
        assert ctx.get(dut.bram_port.en) == 1
        pass

    sim = Simulator(m)
    sim.add_clock(1e-6)

    with sim.write_vcd("output.vcd"):
        sim.run_until(1e-6 * 1000)


def test_hub75():
    m = Module()
    m.submodules.dut = dut = Hub75Coordinator(1)
